@@ -1,21 +1,27 @@
-## Copyright (C) 2000-2017 Daniel Calvelo
-## Copyright (C) 2009 Jaroslav Hajek
+########################################################################
+##
+## Copyright (C) 2000-2023 The Octave Project Developers
+##
+## See the file COPYRIGHT.md in the top-level directory of this
+## distribution or <https://octave.org/copyright/>.
 ##
 ## This file is part of Octave.
 ##
-## Octave is free software; you can redistribute it and/or modify it
+## Octave is free software: you can redistribute it and/or modify it
 ## under the terms of the GNU General Public License as published by
-## the Free Software Foundation; either version 3 of the License, or (at
-## your option) any later version.
+## the Free Software Foundation, either version 3 of the License, or
+## (at your option) any later version.
 ##
 ## Octave is distributed in the hope that it will be useful, but
 ## WITHOUT ANY WARRANTY; without even the implied warranty of
-## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-## General Public License for more details.
+## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+## GNU General Public License for more details.
 ##
 ## You should have received a copy of the GNU General Public License
 ## along with Octave; see the file COPYING.  If not, see
-## <http://www.gnu.org/licenses/>.
+## <https://www.gnu.org/licenses/>.
+##
+########################################################################
 
 ## -*- texinfo -*-
 ## @deftypefn  {} {[@var{s}, @var{i}] =} sortrows (@var{A})
@@ -45,12 +51,9 @@
 ## @seealso{sort}
 ## @end deftypefn
 
-## Author: Daniel Calvelo, Paul Kienzle
-## Adapted-by: jwe
-
 function [s, i] = sortrows (A, c)
 
-  if (nargin < 1 || nargin > 2)
+  if (nargin < 1)
     print_usage ();
   endif
 
@@ -65,7 +68,7 @@ function [s, i] = sortrows (A, c)
   default_mode = "ascend";
   reverse_mode = "descend";
 
-  if (issparse (A))
+  if (issparse (A) || iscell (A))
     ## FIXME: Eliminate this case once __sort_rows_idx__ is fixed to
     ##        handle sparse matrices.
     if (nargin == 1)
@@ -97,7 +100,7 @@ function i = sort_rows_idx_generic (default_mode, reverse_mode, m, c)
     indices = [1:columns(m)]';
     mode(1:columns(m)) = {default_mode};
   else
-    for j = 1:length (c);
+    for j = 1:length (c)
       if (c(j) < 0)
         mode{j} = reverse_mode;
       else
@@ -114,8 +117,12 @@ function i = sort_rows_idx_generic (default_mode, reverse_mode, m, c)
   indices = flipud (indices);
   mode = flipud (mode');
   i = [1:rows(m)]';
-  for j = 1:length (indices);
-    [~, idx] = sort (m(i, indices(j)), mode{j});
+  for j = 1:length (indices)
+    M = m(i, indices(j));
+    if (iscell (M) && ! iscellstr (M))
+      M = cell2mat (M);
+    endif
+    [~, idx] = sort (M, mode{j});
     i = i(idx);
   endfor
 
@@ -144,9 +151,15 @@ endfunction
 %! assert (x, full (sx));
 %! assert (idx, sidx);
 
+%!test <*42523>
+%! C = {1, 2, "filename1";
+%!      3, 4, "filename2";
+%!      5, 6, "filename3"};
+%! C2 = sortrows (C, -1);
+%! assert (C2, flipud (C));
+
 ## Test input validation
-%!error sortrows ()
-%!error sortrows (1, 2, 3)
+%!error <Invalid call> sortrows ()
 %!error sortrows (1, "ascend")
 %!error sortrows (1, ones (2,2))
 %!error sortrows (1, 0)
