@@ -1,6 +1,6 @@
 ########################################################################
 ##
-## Copyright (C) 2010-2021 The Octave Project Developers
+## Copyright (C) 2010-2023 The Octave Project Developers
 ##
 ## See the file COPYRIGHT.md in the top-level directory of this
 ## distribution or <https://octave.org/copyright/>.
@@ -69,7 +69,7 @@ function arg_st = __print_parse_opts__ (varargin)
   arg_st.rgb_output = false;
   arg_st.send_to_printer = false;
   arg_st.special_flag = "textnormal";
-  arg_st.svgconvert = false;
+  arg_st.svgconvert = true;
   arg_st.svgconvert_binary = __quote_path__ (__svgconv_binary__ ());
   arg_st.tight = true;
   arg_st.use_color = 0; # 0=default, -1=mono, +1=color
@@ -89,6 +89,9 @@ function arg_st = __print_parse_opts__ (varargin)
   endif
 
   for i = 1:numel (varargin)
+    if (! ischar (varargin{i}) && ! iscellstr (varargin{i}))
+      error ("print: input arguments must be a graphics handle or strings.");
+    endif
     arg = strtrim (varargin{i});
     if (ischar (arg))
       if (isempty (arg))
@@ -105,6 +108,10 @@ function arg_st = __print_parse_opts__ (varargin)
         arg_st.force_solid = -1;
       elseif (any (strcmp (arg, {"-opengl", "-painters"})))
         arg_st.renderer = arg(2:end);
+      elseif (strcmp (arg, "-image"))
+        arg_st.renderer = "opengl";
+      elseif (strcmp (arg, "-vector"))
+        arg_st.renderer = "painters";
       elseif (strcmp (arg, "-RGBImage"))
         arg_st.rgb_output = true;
         arg_st.renderer = "opengl";
@@ -118,6 +125,8 @@ function arg_st = __print_parse_opts__ (varargin)
         arg_st.tight = true;
       elseif (strcmp (arg, "-svgconvert"))
         arg_st.svgconvert = true;
+      elseif (strcmp (arg, "-nosvgconvert"))
+        arg_st.svgconvert = false;
       elseif (strcmp (arg, "-textspecial"))
         arg_st.special_flag = "textspecial";
       elseif (strcmp (arg, "-fillpage"))
@@ -229,11 +238,6 @@ function arg_st = __print_parse_opts__ (varargin)
              arg_st.devopt);
   endif
 
-  ## By default, postprocess svg files using svgconvert.
-  if (strcmp (arg_st.devopt, "svg"))
-    arg_st.svgconvert = true;
-  endif
-
   ## By default, use the "opengl" renderer for all raster outputs
   ## supported by "imwrite".
   fmts = imformats ();
@@ -248,14 +252,7 @@ function arg_st = __print_parse_opts__ (varargin)
 
   if (strcmp (arg_st.renderer, "auto"))
     if (opengl_ok && strcmp (graphics_toolkit (arg_st.figure), "qt"))
-      ## "opengl" renderer only does text rotations of 0°, 90°, 180°, 270°, ...
-      ht = findall (arg_st.figure, "type", "text");
-      angles = [get(ht, "rotation"){:}];
-      if (any (mod (angles, 90)))
-        arg_st.renderer = "painters";
-      else
-        arg_st.renderer = "opengl";
-      endif
+      arg_st.renderer = "opengl";
     else
       arg_st.renderer = "painters";
     endif
@@ -424,7 +421,7 @@ function arg_st = __print_parse_opts__ (varargin)
     if (! (arg_st.send_to_printer || arg_st.formatted_for_printing
            || strncmp (arg_st.devopt, "pdf", 3)
            || strncmp (arg_st.devopt, "ps", 2)))
-      error ("print: the '%s' option is only valid for page formats and printers.", arg_st.resize_flag);
+      error ("print: the '%s' option is only valid for page formats and printers", arg_st.resize_flag);
     endif
   endif
 
@@ -491,8 +488,10 @@ function arg_st = __print_parse_opts__ (varargin)
                            6 / get (0, "screenpixelsperinch");
     arg_st.ghostscript.resolution = 72;
     arg_st.ghostscript.papersize = arg_st.canvas_size;
+    papersize_points = arg_st.canvas_size * 72;
     arg_st.ghostscript.epscrop = true;
     arg_st.ghostscript.pageoffset = [0, 0];
+    paperposition = [0, 0];
   endif
 
   if (arg_st.formatted_for_printing)
@@ -523,7 +522,7 @@ endfunction
 
 
 ## Test blocks are not allowed (and not needed) for private functions
-#%!test
+%!#test
 %! opts = __print_parse_opts__ ();
 %! assert (opts.devopt, "pswrite");
 %! assert (opts.use_color, 1);
@@ -531,11 +530,11 @@ endfunction
 %! assert (opts.canvas_size, [576, 432]);
 %! assert (opts.ghostscript.device, "pswrite");
 
-#%!test
+%!#test
 %! opts = __print_parse_opts__ ("test.pdf", "-S640,480");
 %! assert (opts.canvas_size, [307.2, 230.4], 0.1);
 
-#%!test
+%!#test
 %! opts = __print_parse_opts__ ("-dpsc", "-append", "-loose");
 %! assert (opts.devopt, "pswrite");
 %! assert (opts.send_to_printer, true);
@@ -544,14 +543,14 @@ endfunction
 %! assert (opts.ghostscript.device, "pswrite");
 %! assert (opts.ghostscript.epscrop, false);
 
-#%!test
+%!#test
 %! opts = __print_parse_opts__ ("-deps", "-tight");
 %! assert (opts.tight, true);
 %! assert (opts.send_to_printer, true);
 %! assert (opts.use_color, -1);
 %! assert (opts.ghostscript.device, "");
 
-#%!test
+%!#test
 %! opts = __print_parse_opts__ ("-djpg", "foobar", "-mono", "-loose");
 %! assert (opts.devopt, "jpeg");
 %! assert (opts.name, "foobar.jpg");
@@ -563,7 +562,7 @@ endfunction
 %! assert (opts.printer, "");
 %! assert (opts.use_color, -1);
 
-#%!test
+%!#test
 %! opts = __print_parse_opts__ ("-ddeskjet", "foobar", "-mono", "-Pmyprinter");
 %! assert (opts.ghostscript.output, "foobar.deskjet");
 %! assert (opts.ghostscript.device, "deskjet");
@@ -572,7 +571,7 @@ endfunction
 %! assert (opts.printer, "-Pmyprinter");
 %! assert (opts.use_color, -1);
 
-#%!test
+%!#test
 %! opts = __print_parse_opts__ ("-f5", "-dljet3");
 %! assert (opts.ghostscript.device, "ljet3");
 %! assert (strfind (opts.ghostscript.output, ".ljet3"));
@@ -639,20 +638,18 @@ function bin = __svgconv_binary__ ()
   persistent binary = "";
 
   if (isempty (binary))
+    ## default installation location is the archlib directory
     bindir = getenv ("OCTAVE_ARCHLIBDIR");
     if (isempty (bindir))
       bindir = __octave_config_info__ ("archlibdir");
     endif
 
-    binary = fullfile (bindir, "octave-svgconvert");
+    binary = fullfile (bindir, ...
+                       ["octave-svgconvert", ...
+                        __octave_config_info__("EXEEXT")]);
 
     if (! exist (binary, "file"))
-      if (! isunix () && exist ([binary, ".exe"], "file"))
-        ## Unix - Includes Mac OSX and Cygwin.
-        binary = [binary, ".exe"];
-      else
-        binary = "";
-      endif
+      binary = "";
     endif
   endif
 
@@ -769,6 +766,7 @@ function value = convert2points (value, units)
 endfunction
 
 function device_list = gs_device_list ()
+
   ## Graphics formats/languages, not printers.
   device_list = {"bmp16"; "bmp16m"; "bmp256"; "bmp32b"; "bmpgray"; ...
                  "epswrite"; "eps2write"; "jpeg"; "jpegcymk"; "jpeggray";
